/* Sample application to demonstrate FT800 primitives, widgets and customized screen shots */

#include "FT_Platform.h"
#include "SampleApp.h"

#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)

/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle = 548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;
ft_char8_t FT_DispCSpread = 1;
ft_char8_t FT_DispDither = 1;

/* Global used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;

ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif
/* Boot up for FT800 followed by graphics primitive sample cases */
/* Initial boot up DL - make the back ground green color */
const ft_uint8_t FT_DLCODE_BOOTUP[12] = 
{
    0,0,0,2,//GPU instruction CLEAR_COLOR_RGB
    7,0,0,38, //GPU instruction CLEAR
    0,0,0,0,  //GPU instruction DISPLAY
};
ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif

   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination

  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);
#endif  
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION
   if (Ft_DlBuffer_Index > 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif
   Ft_DlBuffer_Index = 0;

}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif     
   Ft_CmdBuffer_Index = 0;
}

/* API to check the status of previous DLSWAP and perform DLSWAP of new DL */
/* Check for the status of previous DLSWAP and if still not done wait for few ms and check again */
ft_void_t SAMAPP_GPU_DLSwap(ft_uint8_t DL_Swap_Type)
{
	ft_uint8_t Swap_Type = DLSWAP_FRAME,Swap_Done = DLSWAP_FRAME;

	if(DL_Swap_Type == DLSWAP_LINE)
	{
		Swap_Type = DLSWAP_LINE;
	}

	/* Perform a new DL swap */
	Ft_Gpu_Hal_Wr8(phost,REG_DLSWAP,Swap_Type);

	/* Wait till the swap is done */
	while(Swap_Done)
	{
		Swap_Done = Ft_Gpu_Hal_Rd8(phost,REG_DLSWAP);

		if(DLSWAP_DONE != Swap_Done)
		{
			Ft_Gpu_Hal_Sleep(10);//wait for 10ms
		}
	}
}

ft_void_t SAMAPP_BootupConfig()
{
    Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);
		/* Access address 0 to wake up the FT800 */
		Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);
		Ft_Gpu_Hal_Sleep(20);
		{
			ft_uint8_t chipid;
			//Read Register ID to check if FT800 is ready.
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
			while(chipid != 0x7C)
			{
				chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
				ft_delay(100);
			}	
			printf("VC1 register ID after wake up %x\n",chipid);	
	}

	Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
	Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
	Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
	Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
	Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);
	Ft_Gpu_Hal_Wr16(phost, REG_CSPREAD, FT_DispCSpread);
	Ft_Gpu_Hal_Wr16(phost, REG_DITHER, FT_DispDither);

    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,RESISTANCE_THRESHOLD);

	Ft_Gpu_Hal_Wr16(phost, REG_GPIOX_DIR, 0xffff);
	Ft_Gpu_Hal_Wr16(phost, REG_GPIOX, 0xffff);

    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);


    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD
    ft_delay(120);

	/* make the spi to quad mode - addition 2 bytes for silicon */
	/* api to set quad and numbe of dummy bytes */
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_SINGLE_CHANNEL,FT_GPU_SPI_ONEDUMMY);

phost->ft_cmd_fifo_wp = Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE);

}

ft_void_t SAMAPP_CoPro_Widget_Calibrate()
{
	ft_uint8_t *pbuff;
	ft_uint32_t NumBytesGen = 0, TransMatrix[6];
	ft_uint16_t CurrWriteOffset = 0;

	/*************************************************************************/
	/* Below code demonstrates the usage of calibrate function. Calibrate    */
	/* function will wait untill user presses all the three dots. Only way to*/
	/* come out of this api is to reset the coprocessor bit.                 */
	/*************************************************************************/
	{
		Ft_Gpu_CoCmd_Dlstart(phost);//Start a display list
		Ft_App_WrCoCmd_Buffer(phost, CLEAR_COLOR_RGB(64, 64, 64));// Set the "Clear screen" color default to dark grey
		Ft_App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));// Clear the screen to black
		Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 255, 255)); //Set the drawing color to white		
		Ft_Gpu_CoCmd_Text(phost, (FT_DispWidth / 2), (FT_DispHeight / 2), 27, OPT_CENTER, "Please Tap on the dot");// In the center of the screen, write "Please Tap On dot"
		Ft_Gpu_CoCmd_Calibrate(phost, 0);// Send a "Calibrate" command to the EVE2 to initiate a calibration sequence		
		Ft_App_Flush_Co_Buffer(phost);// Send the display list into the FIFO buffer		
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);//Wait until the co-processor completers the calibration sequence		
		Ft_Gpu_Hal_RdMem(phost, REG_TOUCH_TRANSFORM_A, (ft_uint8_t *)TransMatrix, 4 * 6); //read all 6 touch calibration values.
	}
}

ft_int32_t main(ft_int32_t argc, ft_char8_t *argv[])
{
	Ft_Gpu_HalInit_t halinit;
	ft_int32_t tagValue = 0, i = 0;
	halinit.TotalChannelNum = 1;//Set how many channels to use for communication

	Ft_Gpu_Hal_Init(&halinit);
	host.hal_config.channel_no = 0;//Set the channel number

	host.hal_config.pdn_pin_no = 7;// Specify the power down pin
	host.hal_config.spi_cs_pin_no = 0;// Specify the chip select pin
	host.hal_config.spi_clockrate_khz = 12000; //Specify the clock rate in KHz

	Ft_Gpu_Hal_Open(&host);
	phost = &host;

	SAMAPP_BootupConfig();// Configure the display settings

	Ft_Gpu_Hal_Sleep(1000); //Show the booting up screen.

	SAMAPP_CoPro_Widget_Calibrate();//Run the Touch Screen Calibration sequence	

	while (tagValue != 2)//While the "Exit" button has not been pressed, run this loop
	{
		/*************************************************************************************
		 *The following loop visibly showcases the process of drawing the Matrix Orbital Logo*
		 *The loop is designed to show which new primitive is being drawn at each iteration. *
		 *************************************************************************************/
		for (i = 0; i < 6; i++){
			// First, draw a white rectangle as the background of the logo
			Ft_Gpu_CoCmd_Dlstart(phost);// Start a new Display List
			Ft_App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));// Clear the current screen to black
			Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 255, 255));// Set the drawing color to white
			Ft_App_WrCoCmd_Buffer(phost, BEGIN(RECTS));// Begin drawing a Rectangle Primitive
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(140 * 16, 35 * 16));// Specify the coordinates for the Top Left corner of the rectangle
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(340 * 16, 140 * 16));// Specify the coordinates for the Bottom Right corner of the rectangle
			if (i == 0){
				printf("Drawing Rectangle\n");
			}
			
			// Crop the image by limiting where data can be drawn
			if (i >= 5){
				Ft_App_WrCoCmd_Buffer(phost, SCISSOR_XY(150, 35));// Set the X and Y coordinates of the rectangle clip
				Ft_App_WrCoCmd_Buffer(phost, SCISSOR_SIZE(190, 105));// Set the Height and width of the rectangle clip				
				if (i == 5){
					printf("Clipping the image\n");
				}
			}
			
			// Draw a yellow Circle
			if (i >= 1){
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(250, 250, 0));// Set the drawing color to yellow
				Ft_App_WrCoCmd_Buffer(phost, POINT_SIZE(100 * 16));// Set the radius of the circle that will be drawn
				Ft_App_WrCoCmd_Buffer(phost, BEGIN(FTPOINTS));// Begin drawing a Circle Primitive
				Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(240 * 16, 160 * 16));// Set the X and Y coordinates for the centre of the circle
				if (i == 1){
					printf("Drawing Yellow Circle\n");
				}
			}

			// Overlap a white circle over the yellow circle to form a half moon
			if (i >= 2){
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 255, 255));// Set the drawing color to white
				Ft_App_WrCoCmd_Buffer(phost, POINT_SIZE(135 * 16));// Set the radius of the circle that will be drawn
				Ft_App_WrCoCmd_Buffer(phost, BEGIN(FTPOINTS));// Begin drawing a Circle Primitive
				Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(213 * 16, 190 * 16));// Set the X and Y coordinates for the centre of the circle
				if (i == 2){
					printf("Drawing White Circle\n");
				}
			}

			// Draw a blue Circle
			if (i >= 3){
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(0, 0, 255));// Set the drawing color to blue
				Ft_App_WrCoCmd_Buffer(phost, POINT_SIZE(20 * 16));// Set the radius of the circle that will be drawn
				Ft_App_WrCoCmd_Buffer(phost, BEGIN(FTPOINTS));// Begin drawing a Circle Primitive
				Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(315 * 16, 60 * 16));// Set the X and Y coordinates for the centre of the circle
				if (i == 3){
					printf("Drawing Blue Circle\n");
				}
			}

			// Write "Matrix Orbital" on screen
			if (i >= 4){
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(0, 0, 0));// Set the drawing color to black
				Ft_App_WrCoCmd_Buffer(phost, CMD_TEXT);// Start sending text to the display
				Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)80 << 16) | (210 & 0xffff))); // Specify the Y and X coordinates
				Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)OPT_CENTER << 16) | (31 & 0xffff)));// Set the options and font handle
				Ft_App_WrCoStr_Buffer(phost, "Matrix");// Text string that will be written on the display				
				Ft_App_WrCoCmd_Buffer(phost, CMD_TEXT);// Start sending text to the display
				Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)120 << 16) | (230 & 0xffff)));// Specify the Y and X coordinates
				Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)OPT_CENTER << 16) | (31 & 0xffff)));// Set the options and font handle
				Ft_App_WrCoStr_Buffer(phost, "Orbital");// Text string that will be written on the display
				if (i == 4){
					printf("Writing Text\n");
				}
			}

			Ft_App_WrCoCmd_Buffer(phost, DISPLAY());// Let the co-processor know that the display list is complete
			Ft_Gpu_CoCmd_Swap(phost);// Swap the coprocessor command frames

			Ft_App_Flush_Co_Buffer(phost);// Download the commands into fifo 

			Ft_Gpu_Hal_WaitCmdfifo_empty(phost);// Wait till coprocessor completes the operation
			Ft_Gpu_Hal_Sleep(400);
		}
		
		/***********************************************************************************************************
		 *Once the logo is complete, a snapshot is taken and will be used in the button loop to ensure the logo is *
		 *displayed until the user determines what to do next                                                      * 
		 ***********************************************************************************************************/

		//Take a snapshot of the matrix Orbital logo drawn on screen
		Ft_App_WrCoCmd_Buffer(phost, CMD_SNAPSHOT2);// Start taking a snapshot on the screen, and store it in the graphics memory
		Ft_App_WrCoCmd_Buffer(phost, RGB565);// Specify the snapshot format. Capture the snapshot in RGB565 format
		Ft_App_WrCoCmd_Buffer(phost, RAM_G);// Snapshot address in the graphics RAM
		Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)35 << 16) | (140 & 0xffff)));// Specify the Yand X coordinate of the top left of your snapshot
		Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)105 << 16) | (200 & 0xffff)));// Specify the Height and Width of the snapshot
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);//Wait till coprocessor completes the operation
		Ft_Gpu_Hal_Sleep(100);

		tagValue = Ft_Gpu_Hal_Rd8(phost, REG_TOUCH_TAG);// Read the touch tag registerfor any touch tag inputs

		/***********************************************************************************************************
		 *This loop will continuously draw two buttons, and the primitive base Matrix Orbital logo until the user  *
		 *decides what to do                                                                                       *
		 ***********************************************************************************************************/
		while (!tagValue)//While a button has not been pressed, run this loop
		{
			tagValue = Ft_Gpu_Hal_Rd8(phost, REG_TOUCH_TAG);// Read for any new touch tag inputs
			Ft_Gpu_CoCmd_Dlstart(phost);// Start a new display list
			Ft_App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));// Begin drawing a bitmap					
			Ft_App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));// Clear the current screen
			Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 255, 255));//Set the current drawing color to white

			//Draw the Matrix Orbital Logo snapshot in the center of the screen
			Ft_App_WrCoCmd_Buffer(phost, BITMAP_SOURCE(0));// Specify the source of the bitmap
			Ft_App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT(RGB565, 200 * 2, 105));// Specify the memory format and layout of the bitmap
			Ft_App_WrCoCmd_Buffer(phost, BITMAP_SIZE(NEAREST, BORDER, BORDER, 200, 105));// Specify what filter to apply, wrap to apply around the image, and the size on screen
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(140 * 16, 35 * 16));// Specify the X and Y coordinate for the top left coordinate of the bitmap
			Ft_App_WrCoCmd_Buffer(phost, END());// End the bitmap display list

			/*******************************************************************************************************
			 *When pressed, the program will exit the button loop, and redraw the matrix orbital logo once again   *
			 * using primitives                                                                                    *
			 *******************************************************************************************************/
			//Create a "Replay" Button
			Ft_Gpu_CoCmd_FgColor(phost, 16777215);// Set the foreground color of the button
			Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(0, 0, 255));// Set the drawing color for the button text
			Ft_App_WrCoCmd_Buffer(phost, TAG(1));// Set the Tag ID for the button
			Ft_App_WrCoCmd_Buffer(phost, CMD_BUTTON);// Start creating a button
			Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)150 << 16) | (140 & 0xffff)));// Set the Y and X coordinates for the button
			Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)50 << 16) | (200 & 0xffff)));// Set the height and width of the button
			Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)0 << 16) | (25 & 0xffff)));// Set the Options and Font used by the button
			Ft_Gpu_CoCmd_SendStr(phost, "Replay");// Specify the button string

			/*******************************************************************************************************
			*When pressed, the program will exit the loop and end the program                                      *
			********************************************************************************************************/
			/*Create an Exit Button*/
			Ft_Gpu_CoCmd_FgColor(phost, 16777215);// Set the foreground color 
			Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 0, 0));// Set the drawing color for the button text
			Ft_App_WrCoCmd_Buffer(phost, TAG(2));// Set the Tag for the button
			Ft_App_WrCoCmd_Buffer(phost, CMD_BUTTON);// Start creating a button
			Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)210 << 16) | (140 & 0xffff)));// Set the Y and X coordinates for the button
			Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)50 << 16) | (200 & 0xffff)));// Set the height and width of the button
			Ft_App_WrCoCmd_Buffer(phost, (((ft_uint32_t)0 << 16) | (25 & 0xffff)));// Set the Options and Font used by the button
			Ft_Gpu_CoCmd_SendStr(phost, "Exit");// Specify the button string

			Ft_App_WrCoCmd_Buffer(phost, DISPLAY());// End the Display list
			Ft_Gpu_CoCmd_Swap(phost);// Swap the screens

			Ft_App_Flush_Co_Buffer(phost);//Download the commands into fifo

			Ft_Gpu_Hal_WaitCmdfifo_empty(phost);//Wait till coprocessor completes the operation
			Ft_Gpu_Hal_Sleep(50);
		}

		//After the user presses a button, the display is cleared
		Ft_Gpu_CoCmd_Dlstart(phost);// Start a new display list
		Ft_App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));// Clear the current screen
		Ft_App_WrCoCmd_Buffer(phost, DISPLAY());// End the Display list
		Ft_Gpu_CoCmd_Swap(phost);// Swap the screens

		Ft_App_Flush_Co_Buffer(phost);//Download the commands into fifo

		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);//Wait till coprocessor completes the operation
		Ft_Gpu_Hal_Sleep(50);				
	}
	
/* Close all the opened handles */
Ft_Gpu_Hal_Close(phost);
Ft_Gpu_Hal_DeInit();

return 0;

}
