/******************************************************
*Matrix Orbital Logo using primitives Rev1.0          *
*SampleApp taken from FTDI Chip                       *
*Modified by Daniel Divino, 24/07/2017                *
*support@matrixorbital.ca                             *
*www.matrixorbital.ca/appnote                         *
*LCDforums.ca                                         *
******************************************************/

/**********************************************************************************************************************************************
* MO: Since this example is a modified version of FTDI's SampleApp, we (Matrix Orbital) have taken the liberty of commenting and providing    *
*     as much detail and explanation as possible to ensure that the user understands each function and command in the code.                   *
*     Comments annotated with "MO:" are made by Matrix Orbital. All other comments were made by FTDI.                                         *
*     We started with FTDI's SampleApp, and used the libraries FTDI has provided to modify the code. Certain functions have been retained,    *
*  	  such as their write and read functions, their calibration function, and their bootup/configuration functions.                           *
*     All modifications to their code are found in the SampleApp.c file. No other changes have been made to the rest of the files included in *
*     the application.                                                                                                                        *
*     Many ifdef statements can be found in the code space. This is to allow for compatibility with future controllers, such as Arduino, and  *
*     FTDI's FT900 series.                                                                                                                    *
**********************************************************************************************************************************************/

/* MO: Sample application to demonstrate FT81X primitives, buttons and customized screen shots */

#include "FT_Platform.h"
#include "SampleApp.h"
#if defined(ARDUINO_PLATFORM) 
#include<SPI.h>
#endif

#define SAMAPP_DELAY_BTW_APIS (1000)// MO: Delay for 1 second
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS) //Delay  
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x) //Delay for a certain amount of milliseconds

#define EVE2_43
//#define EVE2_70
//#define EVE2_50
//#define EVE2_38
//#define EVE2_35
//#define EVE2_29

/* Global variables for display resolution to support various display panels */
#if defined(EVE2_43)
ft_int16_t FT_DispWidth = 480; // MO: Specify the horizontal resolution of the display
ft_int16_t FT_DispHeight = 272;// MO: Specify vertical resolution of the display
ft_int16_t FT_DispHCycle = 548;// MO: Specify the number of PCLK clock edges needed to scan a horizontal line
ft_int16_t FT_DispHOffset = 43;// MO: How many PCLK clock edges are sent before the horizontal line's pixels are scanned out
ft_int16_t FT_DispHSync1 = 41; // MO: How many PCLK clock edges are used for syncing the display
ft_int16_t FT_DispHSync0 = 0;  // MO: How many HSYNC PCLK edges are high state when syncing
ft_int16_t FT_DispVCycle = 292;// MO: How many vertical pixel lines are scanned in one frame
ft_int16_t FT_DispVOffset = 12;// MO: How many clock  edges are sent before the vertical line's pixels are scanned out
ft_int16_t FT_DispVSync1 = 10; // MO: How many PCLK edges are needed to synchronize the vertical timing of a new frame
ft_int16_t FT_DispVSync0 = 0;  // MO: How many PCLK edges are high state during vertical synchronization
ft_uint8_t FT_DispPCLK = 5;    // MO: The PCLK will be divided by this value. Ie. if the main clock is 60MHz, and a DispPCLK of 5, the PCLK becomes 12MHz
ft_char8_t FT_DispSwizzle = 0; // MO: Set the RGB pin mapping. Please check your display data sheet for RGB pin layout
ft_char8_t FT_DispPCLKPol = 1; // MO: Specify the polarity of the display clock cycles
ft_char8_t FT_DispCSpread = 1; // MO: This bit helps control the transition of RGB signals. if 0, all color signals are updated at the same time. If 
//     1, the timings are adjusted slightly to reduce the number of simultaneous signal changes. Setting to 1 also reduces noise.                              
ft_char8_t FT_DispDither = 1;  // MO: Use this register to enable/disable the dithering feature on the RGB signal's output.*/
#endif
#if defined(EVE2_70) || defined(EVE2_50)
ft_int16_t FT_DispWidth = 800;
ft_int16_t FT_DispHeight = 480;
ft_int16_t FT_DispHCycle = 928;
ft_int16_t FT_DispHOffset = 88;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 48;
ft_int16_t FT_DispVCycle = 525;
ft_int16_t FT_DispVOffset = 32;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 3;
ft_int16_t FT_DispPCLK = 2;
ft_int16_t FT_DispSwizzle = 0;
ft_int16_t FT_DispPCLKPol = 1;
ft_int16_t FT_DispCSpread = 0;
ft_int16_t FT_DispDither = 1;
#endif
#if defined(EVE2_38)
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 116;
ft_int16_t FT_DispHCycle = 524;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispVCycle = 288;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 2;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;
ft_char8_t FT_DispCSpread = 1;
ft_char8_t FT_DispDither = 1;
#endif
#if defined(EVE2_35)
ft_int16_t FT_DispWidth = 320;
ft_int16_t FT_DispHeight = 240;
ft_int16_t FT_DispHCycle = 408;
ft_int16_t FT_DispHOffset = 70;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 10;
ft_int16_t FT_DispVCycle = 262;
ft_int16_t FT_DispVOffset = 18;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 2;
ft_int16_t FT_DispPCLK = 8;
ft_int16_t FT_DispSwizzle = 0;
ft_int16_t FT_DispPCLKPol = 0;
ft_int16_t FT_DispCSpread = 1;
ft_int16_t FT_DispDither = 1;
#endif
#if defined(EVE2_29)
ft_int16_t FT_DispWidth = 320;
ft_int16_t FT_DispHeight = 102;
ft_int16_t FT_DispHCycle = 408;
ft_int16_t FT_DispHOffset = 70;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 10;
ft_int16_t FT_DispVCycle = 262;
ft_int16_t FT_DispVOffset = 156;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 2;
ft_int16_t FT_DispPCLK = 8;
ft_int16_t FT_DispSwizzle = 1;
ft_int16_t FT_DispPCLKPol = 0;
ft_int16_t FT_DispCSpread = 1;
ft_int16_t FT_DispDither = 1;
#endif


/* Globals used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;
ft_uint32_t Ft_CmdBuffer_Index;// MO: Initialize the index of the coprocessor command buffer
ft_uint32_t Ft_DlBuffer_Index; // MO: Initialize the index of the display list buffer

/***************************************************************************************************
* MO: The Buffer optimization listed below affects how data will be sent to the EVE2 module.       *
*     If enabled, all Coprocessor and Display list commands will be stored in their own respective *
*     buffers. When the EVE is ready to be updated, all the data in the Coprocessor command buffer *
*     and Display List buffer will be sent off to the display as a chunk of data.                  *                                                                                            *
*     If Buffer Optimization is disabled, every command will be sent individually to the display.  *
***************************************************************************************************/
#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];       // MO: Initialize thecommand buffer
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];// MO: Initialize the Display list buffer
#endif

/***************************************************************************************************
* MO: The DLCODE_BOOTUP array featured below takes the raw byte values for 3 instructions;         *
*     the CLEAR_COLOR_RGB, CLEAR, and DISPLAY instructions, and places them all into one function. *
*     By doing this, a user will be able to simply write this array of bytes to the EVE2, and the  *
*     screen will be cleared immediately.                                                          *
***************************************************************************************************/

/* Boot up for FT81X followed by graphics primitive sample cases */
/* Initial boot up DL - make the back ground green color */
const ft_uint8_t FT_DLCODE_BOOTUP[12] =
{
    0,0,0,2,//GPU instruction CLEAR_COLOR_RGB
    7,0,0,38, //GPU instruction CLEAR
    0,0,0,0,  //GPU instruction DISPLAY
};

/********************************************************************************************************************
* MO: This function places the specified command into the Command buffer, to be sent to the display.                *
*     Commands written in the WrCoCmd buffer are sent to the EVE Module's co-processor to be drawn on screen.       *
*     All commands are in 4 bytes in length, and thus, the EVE2 module's memory will always have to be incremented  *
*     by 4 bytes.                                                                                                   *
********************************************************************************************************************/
ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;                                     // MO: Initialize a 32 bit pointer
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index]; // MO: Take the address of the next 4 bits in the buffer
   *pBuffcmd = cmd;                                           // MO: Place the command at the address taken above
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);// MO: Send the command to the display directly
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);// MO: Send the command to the display directly
#endif
   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;
}

/* MO: This function places the specified command into the Display List buffer, to be sent to the display 
	   Commands written in the WrCoCmd buffer are sent to the EVE Module's to be drawn on screen */
ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;                                   // MO: Initialize a 32 bit pointer
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index]; // MO: Take the address of the next 4 bits in the buffer
   *pBuffcmd = cmd;                                         // MO: Place the command at the address taken above
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);// MO: Send the command to the display directly
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);// MO: Send the command to the display directly
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;// MO: Initialize a length variable for strings.
  length = strlen(s) + 1;//last for the null termination
						 // MO: Obtain the length of the string + 1 for the null terminator 						 
  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s); // MO: Copy the string and place it in the command buffer.

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3); 
#endif  
}

/* MO: This function sends all the commands stored in the Display list buffer to the EVE2 module */
ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION
   if (Ft_DlBuffer_Index > 0)									  // MO: If there are any commands in the DL buffer, Write the data to the display
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);// MO: Write all the data from the Display List Buffer to the display
#endif
   Ft_DlBuffer_Index = 0;// MO: Set the buffer index to 0.

}

/* MO: This function sends all the commands stored in the Coprocessor Command buffer to the EVE2 module */
ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)                                  // MO: If there are any commands in the command buffer, write it to the display
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);// MO: Write all the data from the Coprocessor Command Buffer to the display
#endif     
   Ft_CmdBuffer_Index = 0;// MO: Set the buffer index to 0
}

/* API to demonstrate calibrate widget/functionality */
ft_void_t SAMAPP_CoPro_Widget_Calibrate()
{	
	ft_uint32_t NumBytesGen = 0;
	ft_uint16_t CurrWriteOffset = 0;

	/*************************************************************************/
	/* Below code demonstrates the usage of calibrate function. Calibrate    */
	/* function will wait untill user presses all the three dots. Only way to*/
	/* come out of this api is to reset the coprocessor bit.                 */
	/*************************************************************************/	
	Ft_Gpu_CoCmd_Dlstart(phost);                                                                          // MO: Start a new Display List. This command is required before sending any DL commands to the display.
																										  //     This command will set the current EVE2's coprocessor write location to 0. This prevents overlap 
																										  //     and helps to keep track of where commands are being sent to when starting a new screen.
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(64,64,64));                                               // MO: Set the color to use when clearing the screen.
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));                                                            // MO: Clear the screen.
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));                                                  // MO: Set the color to use for drawing.
	Ft_Gpu_CoCmd_Text(phost,(FT_DispWidth/2), (FT_DispHeight/2), 27, OPT_CENTER, "Please Tap on the dot");// MO: Write Calibration instructions in the center of the screen.
	Ft_Gpu_CoCmd_Calibrate(phost, 0);                                                                     // MO: Request to calibrate the screen.

	/* Download the commands into FIFIO */
	Ft_App_Flush_Co_Buffer(phost);
	/* Wait till coprocessor completes the operation */
	Ft_Gpu_Hal_WaitCmdfifo_empty(phost);	
}

ft_void_t SAMAPP_BootupConfig()
{
    Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);
	Ft_Gpu_Hal_Sleep(100);

		/* Access address 0 to wake up the FT800 */
		Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);// MO: Set the display to Active Status
		Ft_Gpu_Hal_Sleep(100);

		{
			#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
			printf("Reading Chip ID\n");// MO: Print text to the console window
			#endif		
			#if defined(ARDUINO_PLATFORM)
			Serial.print("Reading Chip ID...\n");
			#endif	
			ft_uint8_t chipid;
			//Read Register ID to check if FT800 is ready.
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);// MO: Read the Register ID to check if the display is ready for commands

			#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
			printf("Chip ID: %x\n", chipid);
			#endif		
			#if defined(ARDUINO_PLATFORM)
			Serial.write("Chip ID: ");
			Serial.print(chipid, HEX);
			Serial.write("\n");
			#endif	

			while(chipid != 0x7C)                  // MO: If the display is not ready, continuously read the register until the display is ready
			{
				chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);// MO: Read the Register ID to check if the display is ready for commands
				#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
				printf("Chip ID: %x\n", chipid);
				#endif		
				#if defined(ARDUINO_PLATFORM)
				Serial.write("Chip ID: ");
				Serial.print(chipid, HEX);
				Serial.write("\n");
				#endif
				ft_delay(100);
			}
		}
	
	/* Configuration of LCD display */
	// MO: The following block will configure the EVE2 module. This block uses the specifications listed at the very beginning of this file to configure the 
    //     EVE2 module for communicatio with the attached display
	Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
	Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
	Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
	Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
	Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);
	Ft_Gpu_Hal_Wr16(phost, REG_CSPREAD, FT_DispCSpread);
	Ft_Gpu_Hal_Wr16(phost, REG_DITHER, FT_DispDither);
	Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH, RESISTANCE_THRESHOLD);
	Ft_Gpu_Hal_Wr16(phost, REG_GPIOX_DIR, 0xffff);
	Ft_Gpu_Hal_Wr16(phost, REG_GPIOX, 0xffff);
    
    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);

    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD
    ft_delay(120);

	/*********************************************************************************************
	* MO: The following ifdef statement is meant to be used by the FT900 processor specifically, *
	*     and is only included to maintain compatibility with future platforms. If the #ifdef    *
	*     statement does not apply to the platform you are using, you may remove it to reduce    *
	*     code length.                                                                           *
	*********************************************************************************************/
	/* Reconfigure the SPI */
#ifdef FT900_PLATFORM
	printf("after ILI9488 bootup \n");
	//spi
	// Initialize SPIM HW
	sys_enable(sys_device_spi_master);
	gpio_function(27, pad_spim_sck); /* GPIO27 to SPIM_CLK */
	gpio_function(28, pad_spim_ss0); /* GPIO28 as CS */
	gpio_function(29, pad_spim_mosi); /* GPIO29 to SPIM_MOSI */
	gpio_function(30, pad_spim_miso); /* GPIO30 to SPIM_MISO */

	gpio_write(28, 1);
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);
#endif

	/* make the spi to quad mode - addition 2 bytes for silicon */
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_SINGLE_CHANNEL,FT_GPU_SPI_ONEDUMMY);

	phost->ft_cmd_fifo_wp = Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE);
}


/* Main entry point */
#if defined(MSVC_PLATFORM) || defined(FT900_PLATFORM)
/* Main entry point */
ft_int32_t main(ft_int32_t argc, ft_char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM) || defined(MSVC_FT800EMU)
ft_void_t setup()
#endif
{	
	#if defined(ARDUINO_PLATFORM) 
	Serial.begin(9600);
	#endif
	ft_int32_t tagValue = 0, i = 0;
	Ft_Gpu_HalInit_t halinit;
	int x1, x2, y1, y2;

	halinit.TotalChannelNum = 1; // MO: Following description taken from FT_GPU_Hal.c
								 //      Total number channels for underlying usb - bridge
								 //      library(libmpsse / LibFT4222

	Ft_Gpu_Hal_Init(&halinit); // MO: Following description taken from FT_GPU_Hal.c
							   //	   API for USB-to-SPI/I2C bridge IC LIB initialize  
							   //      For MSVC_PLATFORM the following actions are
							   //      performed
							   //      -libMPSSE / libFT4222 init
							   //      -read info about SPI channels on usb - bridge chip
							   //      through lib service
	host.hal_config.channel_no = 0;// MO: mpsse channel number

	host.hal_config.pdn_pin_no = FT800_PD_N;    // MO: Set the power down pin.
	host.hal_config.spi_cs_pin_no = FT800_SEL_PIN; // MO: Set the Chip select pin.

	#if defined(MSVC_PLATFORM) && defined(MSVC_PLATFORM_SPI_LIBMPSSE)
	host.hal_config.spi_clockrate_khz = 12000; //in KHz
	#elif defined(MSVC_PLATFORM) && defined(MSVC_PLATFORM_SPI_LIBFT4222)
	host.hal_config.spi_clockrate_khz = 20000; //in KHz
	#endif

	#ifdef ARDUINO_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 4000; //in KHz
	#endif

	Ft_Gpu_Hal_Open(&host); // MO: obtain handle for interface communication
	phost = &host;

	SAMAPP_BootupConfig();// MO: Run Bootup sequence

	/*It is optional to clear the screen here*/
	Ft_Gpu_Hal_WrMem(phost, RAM_DL, (ft_uint8_t *) FT_DLCODE_BOOTUP, sizeof(FT_DLCODE_BOOTUP));
	Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP, DLSWAP_FRAME);

	Ft_Gpu_Hal_Sleep(1000); //Show the booting up screen.

	SAMAPP_CoPro_Widget_Calibrate();// MO: Run the Touch Screen Calibration sequence	

	while (tagValue != 2)// MO: While the "Exit" button has not been pressed, run this loop
	{
		#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
		printf("Drawing Logo\n"); // MO: Print current step to console window
		#endif	
		#if defined(ARDUINO_PLATFORM)
		Serial.print("Drawing Logo\n");// MO: Print current step to console window
		#endif	
		/******************************************************************************************
		* MO: The following loop visibly showcases the process of drawing the Matrix Orbital Logo *
		*     The loop is designed to show which new primitive is being drawn at each iteration.  *
		*     Because the EVE2 updates it's entire screen each frame, primitives cannot be drawn  *
		*     individually. The entire screen must be redrawn. ie. you cannot draw a rectangle in *
		*     one frame, and draw a circle on top in the next. In order to implement this visual  *
		*     effect, you will have to first draw the rectangle for one frame. Then, draw the     *
		*     rectangle and circle on top for the next frame.                                     *
		******************************************************************************************/
		for (i = 0; i < 9; i++) {
			/* MO: First iteration, draw a white rectangle in the center of the screen as the background of the logo */
			Ft_Gpu_CoCmd_Dlstart(phost);                                                                               // MO: Start a new Display List
			Ft_App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));                                                              // MO: Clear the current screen to black. Whenever a new display list is started,
			//     it is recommended that the screen is cleared.
			Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 255, 255));                                                    // MO: Set the drawing color to white
			Ft_App_WrCoCmd_Buffer(phost, BEGIN(RECTS));                                                                // MO: Begin drawing a Rectangle Primitive
			if (FT_DispHeight < 240) {
				x1 = 0;
				x2 = 200;
				y1 = 0;
				y2 = 100;
			}
			else {
				x1 = ((FT_DispWidth / 2) - 100);
				x2 = ((FT_DispWidth / 2) + 100);
				y1 = ((FT_DispHeight / 4) - 50);
				y2 = ((FT_DispHeight / 4) + 50);
			}
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(x1 * 16, y1 * 16)); // MO: Specify the coordinates for the Top Left corner of the rectangle. 
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(x2 * 16, y2 * 16));  // MO: Specify the coordinates for the Bottom Right corner of the rectangle
			if (i == 0) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
				printf("Drawing Rectangle\n"); // MO: Print current step to console window
#endif	
#if defined(ARDUINO_PLATFORM)
				Serial.print("Drawing Rectangle\n"); // MO: Print current step to console window
#endif	
			}
			/* MO: Sixth iteration, limit the area that can be drawn on.
			This will crop the image to ensure the logo is rectangular when completed.*/
			if (i >= 5) {
				if (FT_DispHeight < 240) {
					x1 = 0;
					y1 = 0;
				}
				else {
					x1 = ((FT_DispWidth / 2) - 100);
					y1 = ((FT_DispHeight / 4) - 50);
				}
				Ft_App_WrCoCmd_Buffer(phost, SCISSOR_XY(x1, y1));// MO: Set the X and Y coordinates of the rectangle clip
				Ft_App_WrCoCmd_Buffer(phost, SCISSOR_SIZE(200, 100));                                             // MO: Set the Height and width of the rectangle clip				
				if (i == 5) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Clipping the image\n"); // MO: Print current step to console window
#endif					
#if defined(ARDUINO_PLATFORM)
					Serial.print("Clipping the image\n"); // MO: Print current step to console window
#endif	
				}
			}
			/* MO: Second iteration, draw a yellow Circle */
			if (i >= 1) {
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(250, 250, 0));                                            // MO: Set the drawing color to yellow
				Ft_App_WrCoCmd_Buffer(phost, POINT_SIZE(90 * 16));                                              // MO: Set the radius of the circle that will be drawn
				Ft_App_WrCoCmd_Buffer(phost, BEGIN(FTPOINTS));                                                   // MO: Begin drawing a Circle Primitive			
				if (FT_DispHeight < 240) {
					x1 = 100;
					y1 = 100;
				}
				else {
					x1 = FT_DispWidth / 2;
					y1 = (FT_DispHeight / 4) + 50;
				}
				Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(x1 * 16, y1 * 16));// MO: Set the X and Y coordinates for the centre of the circle
				if (i == 1) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Drawing Yellow Circle\n"); // MO: Print current step to console window
#endif						
#if defined(ARDUINO_PLATFORM)					
					Serial.print("Drawing Yellow Circle\n"); // MO: Print current step to console window
#endif		
				}
			}

			/* MO: Third iteration, Overlap a white circle over the yellow circle to form a half moon */
			if (i >= 2) {
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 255, 255)); // MO: Set the drawing color to white
				Ft_App_WrCoCmd_Buffer(phost, POINT_SIZE(115 * 16));     // MO: Set the radius of the circle that will be drawn
				Ft_App_WrCoCmd_Buffer(phost, BEGIN(FTPOINTS));          // MO: Begin drawing a Circle Primitive
				if (FT_DispHeight < 240) {
					x1 = 75;
					y1 = 120;
				}
				else {
					x1 = ((FT_DispWidth / 2) - 25);
					y1 = ((FT_DispHeight / 4) + 70);
				}
				Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(x1 * 16, y1 * 16));// MO: Set the X and Y coordinates for the centre of the circle
				if (i == 2) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Drawing White Circle\n"); // MO: Print current step to console window
#endif	
#if defined(ARDUINO_PLATFORM)
					Serial.print("Drawing White Circle\n"); // MO: Print current step to console window
#endif	
				}
			}

			/* MO: Fourth iteration, draw a blue Circle */
			if (i >= 3) {
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(0, 0, 255)); // MO: Set the drawing color to blue
				Ft_App_WrCoCmd_Buffer(phost, POINT_SIZE(15 * 16));  // MO: Set the radius of the circle that will be drawn
				Ft_App_WrCoCmd_Buffer(phost, BEGIN(FTPOINTS));      // MO: Begin drawing a Circle Primitive
				if (FT_DispHeight < 240) {
					x1 = 180;
					y1 = 20;
				}
				else {
					x1 = ((FT_DispWidth / 2) + 83);
					y1 = ((FT_DispHeight / 4) - 30);
				}
				Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(x1 * 16, y1 * 16));// MO: Set the X and Y coordinates for the centre of the circle
				if (i == 3) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Drawing Blue Circle\n"); // MO: Print current step to console window
#endif		
#if defined(ARDUINO_PLATFORM)
					Serial.print("Drawing Blue Circle\n"); // MO: Print current step to console window
#endif	
				}
			}

			/* MO: Fifth iteration, write "Matrix Orbital" on screen */
			if (i >= 4) {
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(0, 0, 0)); // MO: Set the drawing color to black			
				if (FT_DispHeight < 240) {
					x1 = 70;
					x2 = 90;
					y1 = 40;
					y2 = 80;
				}
				else {
					x1 = ((FT_DispWidth / 2) - 30);
					x2 = ((FT_DispWidth / 2) - 10);
					y1 = ((FT_DispHeight / 4) - 15);
					y2 = ((FT_DispHeight / 4) + 25);
				}
				Ft_Gpu_CoCmd_Text(phost, x1, y1, 31, OPT_CENTER, "Matrix"); // MO: Write 'Matrix' on screen
				Ft_Gpu_CoCmd_Text(phost, x2, y2, 31, OPT_CENTER, "Orbital");// MO: Write 'Orbital' underneath 'Matrix'
				if (i == 4) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Writing Text\n"); // MO: Print current step to console window
#endif	
#if defined(ARDUINO_PLATFORM)
					Serial.print("Writing Text\n"); // MO: Print current step to console window
#endif	
				}
			}

			/* MO: Seventh iteration, limit the area that can be drawn on. */
			if (i >= 6) {
				if (FT_DispHeight < 240) {
					x1 = 200;
					y1 = 0;
				}
				else {
					x1 = ((FT_DispWidth / 2) - 100);
					y1 = (FT_DispHeight / 4) + 60;

				}
				Ft_App_WrCoCmd_Buffer(phost, SCISSOR_XY(x1, y1));// MO: Set the X and Y coordinates of the rectangle clip
				Ft_App_WrCoCmd_Buffer(phost, SCISSOR_SIZE(200, 110));// MO: Set the Height and width of the rectangle clip				
				if (i == 6) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Writing Text\n"); // MO: Print current step to console window
#endif	
#if defined(ARDUINO_PLATFORM)
					Serial.print("Writing Text\n"); // MO: Print current step to console window
#endif	
				}
			}

			/* MO: Eighth iteration, Create a "Replay" Button" */
			if (i >= 7) {
				Ft_Gpu_CoCmd_FgColor(phost, 16777215);             // MO: Set the foreground color of the button
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(0, 0, 255));// MO: Set the drawing color for the button text
				Ft_App_WrCoCmd_Buffer(phost, TAG(1));              // MO: Set the Tag ID for the button
				if (FT_DispHeight < 240) {
					x1 = 210;
					x2 = 110;
					y1 = 0;
					y2 = 40;
				}
				else {
					x1 = ((FT_DispWidth / 2) - 100);
					x2 = 200;
					y1 = (FT_DispHeight / 4) + 60;
					y2 = 50;
				}
				Ft_Gpu_CoCmd_Button(phost, x1, y1, x2, y2, 25, 0, "Replay"); // MO: Create a 'Replay' button
				if (i == 7) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Drawing Replay Button\n"); // MO: Print current step to console window
#endif	
#if defined(ARDUINO_PLATFORM)
					Serial.print("Drawing Replay Button\n"); // MO: Print current step to console window
#endif	
				}
			}

			/* MO: Ninth iteration, Create a "Exit" Button" */
			if (i >= 8) {
				Ft_Gpu_CoCmd_FgColor(phost, 16777215);             // MO: Set the foreground color 
				Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 0, 0));// MO: Set the drawing color for the button text
				Ft_App_WrCoCmd_Buffer(phost, TAG(2));              // MO: Set the Tag for the button
				if (FT_DispHeight < 240) {
					x1 = 210;
					x2 = 110;
					y1 = 50;
					y2 = 50;
				}
				else {
					x1 = (FT_DispWidth / 2) - 100;
					x2 = 200;
					y1 = FT_DispHeight / 2;
					y2 = 50;
				}
				Ft_Gpu_CoCmd_Button(phost, x1, y1, x2, y2, 25, 0, "Exit");	// MO: Create a 'Exit' button
				if (i == 8) {
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
					printf("Drawing Exit Button\n"); // MO: Print current step to console window
#endif					
#if defined(ARDUINO_PLATFORM)
					Serial.print("Drawing Exit Button\n"); // MO: Print current step to console window
#endif		
				}
			}

			Ft_App_WrCoCmd_Buffer(phost, DISPLAY());// MO: Let the co-processor know that the display list is complete
			Ft_Gpu_CoCmd_Swap(phost);               // MO: Swap the coprocessor command frames
			Ft_App_Flush_Co_Buffer(phost);          // MO: Download the commands into fifo 
			Ft_Gpu_Hal_WaitCmdfifo_empty(phost);    // MO: Wait till coprocessor completes the operation
			Ft_Gpu_Hal_Sleep(400);
		}

		/* MO: Continuously read for touch input. 0 represents no touch detected.
		There were issues with the EVE2 reporting a 255 touch when the Matrix Orbital Logo was pressed
		so we are ignoring TAG 255 as well*/
		tagValue = Ft_Gpu_Hal_Rd8(phost, REG_TOUCH_TAG);// MO: Read for any new touch tag inputs
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
		printf("Waiting for input\n");			   	    // MO: Print text to the console window		
#endif	
#if defined(ARDUINO_PLATFORM)
		Serial.print("Waiting for input\n\n "); // MO: Print current step to console window
#endif		

		while ((tagValue == 0) || (tagValue == 255))
		{
			tagValue = Ft_Gpu_Hal_Rd8(phost, REG_TOUCH_TAG); // MO: Read for any new touch tag inputs									
		}
#if defined(MSVC_PLATFORM)|| defined(MSVC_FT800EMU)|| defined(FT900_PLATFORM)
		printf("Tag value %d\n\n", tagValue);				 // MO: Print text to the console window
#endif		
		//After the user pressesthe 'Replay button, the display is cleared
		Ft_Gpu_CoCmd_Dlstart(phost);                 // MO: Start a new display list
		Ft_App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));// MO: Clear the current screen
		Ft_App_WrCoCmd_Buffer(phost, DISPLAY());     // MO: End the Display list
		Ft_Gpu_CoCmd_Swap(phost);                    // MO: Swap the screens

		Ft_App_Flush_Co_Buffer(phost);// MO: Download the commands into fifo

		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);// MO: Wait till coprocessor completes the operation
		Ft_Gpu_Hal_Sleep(50);
	}

	/* Close all the opened handles */
	Ft_Gpu_Hal_Close(phost);
	Ft_Gpu_Hal_DeInit();
}

void loop() {
}

/* Nothing beyond this */

